// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_COMMON_MEDIA_ROUTER_MOJO_MEDIA_STATUS_MOJOM_H_
#define CHROME_COMMON_MEDIA_ROUTER_MOJO_MEDIA_STATUS_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "chrome/common/media_router/mojo/media_status.mojom-shared.h"
#include "chrome/common/media_router/mojo/media_status.mojom-forward.h"
#include "mojo/public/mojom/base/time.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "chrome/common/media_router/media_status.h"




namespace media_router {
namespace mojom {

class MediaStatusObserverProxy;

template <typename ImplRefTraits>
class MediaStatusObserverStub;

class MediaStatusObserverRequestValidator;


class  MediaStatusObserver
    : public MediaStatusObserverInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Base_ = MediaStatusObserverInterfaceBase;
  using Proxy_ = MediaStatusObserverProxy;

  template <typename ImplRefTraits>
  using Stub_ = MediaStatusObserverStub<ImplRefTraits>;

  using RequestValidator_ = MediaStatusObserverRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kOnMediaStatusUpdatedMinVersion = 0,
  };
  virtual ~MediaStatusObserver() {}

  
  virtual void OnMediaStatusUpdated(const media_router::MediaStatus& status) = 0;
};

class  MediaStatusObserverProxy
    : public MediaStatusObserver {
 public:
  using InterfaceType = MediaStatusObserver;

  explicit MediaStatusObserverProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnMediaStatusUpdated(const media_router::MediaStatus& status) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  MediaStatusObserverStubDispatch {
 public:
  static bool Accept(MediaStatusObserver* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      MediaStatusObserver* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<MediaStatusObserver>>
class MediaStatusObserverStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  MediaStatusObserverStub() {}
  ~MediaStatusObserverStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return MediaStatusObserverStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return MediaStatusObserverStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  MediaStatusObserverRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};








class  MediaStatus {
 public:
  template <typename T>
  using EnableIfSame = std::enable_if_t<std::is_same<MediaStatus, T>::value>;
  using DataView = MediaStatusDataView;
  using Data_ = internal::MediaStatus_Data;
  using PlayState = MediaStatus_PlayState;

  template <typename... Args>
  static MediaStatusPtr New(Args&&... args) {
    return MediaStatusPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static MediaStatusPtr From(const U& u) {
    return mojo::TypeConverter<MediaStatusPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, MediaStatus>::Convert(*this);
  }


  MediaStatus();

  MediaStatus(
      const std::string& title,
      bool can_play_pause,
      bool can_mute,
      bool can_set_volume,
      bool can_seek,
      MediaStatus::PlayState play_state,
      bool is_muted,
      float volume,
      base::TimeDelta duration,
      base::TimeDelta current_time);

  ~MediaStatus();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = MediaStatusPtr>
  MediaStatusPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T, MediaStatus::EnableIfSame<T>* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        MediaStatus::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        MediaStatus::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::MediaStatus_UnserializedMessageContext<
            UserType, MediaStatus::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<MediaStatus::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return MediaStatus::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::MediaStatus_UnserializedMessageContext<
            UserType, MediaStatus::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<MediaStatus::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  std::string title;
  
  bool can_play_pause;
  
  bool can_mute;
  
  bool can_set_volume;
  
  bool can_seek;
  
  MediaStatus::PlayState play_state;
  
  bool is_muted;
  
  float volume;
  
  base::TimeDelta duration;
  
  base::TimeDelta current_time;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};

// The comparison operators are templates, so they are only instantiated if they
// are used. Thus, the bindings generator does not need to know whether
// comparison operators are available for members.
template <typename T, MediaStatus::EnableIfSame<T>* = nullptr>
bool operator<(const T& lhs, const T& rhs);

template <typename T, MediaStatus::EnableIfSame<T>* = nullptr>
bool operator<=(const T& lhs, const T& rhs) {
  return !(rhs < lhs);
}

template <typename T, MediaStatus::EnableIfSame<T>* = nullptr>
bool operator>(const T& lhs, const T& rhs) {
  return rhs < lhs;
}

template <typename T, MediaStatus::EnableIfSame<T>* = nullptr>
bool operator>=(const T& lhs, const T& rhs) {
  return !(lhs < rhs);
}

template <typename StructPtrType>
MediaStatusPtr MediaStatus::Clone() const {
  return New(
      mojo::Clone(title),
      mojo::Clone(can_play_pause),
      mojo::Clone(can_mute),
      mojo::Clone(can_set_volume),
      mojo::Clone(can_seek),
      mojo::Clone(play_state),
      mojo::Clone(is_muted),
      mojo::Clone(volume),
      mojo::Clone(duration),
      mojo::Clone(current_time)
  );
}

template <typename T, MediaStatus::EnableIfSame<T>*>
bool MediaStatus::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->title, other_struct.title))
    return false;
  if (!mojo::Equals(this->can_play_pause, other_struct.can_play_pause))
    return false;
  if (!mojo::Equals(this->can_mute, other_struct.can_mute))
    return false;
  if (!mojo::Equals(this->can_set_volume, other_struct.can_set_volume))
    return false;
  if (!mojo::Equals(this->can_seek, other_struct.can_seek))
    return false;
  if (!mojo::Equals(this->play_state, other_struct.play_state))
    return false;
  if (!mojo::Equals(this->is_muted, other_struct.is_muted))
    return false;
  if (!mojo::Equals(this->volume, other_struct.volume))
    return false;
  if (!mojo::Equals(this->duration, other_struct.duration))
    return false;
  if (!mojo::Equals(this->current_time, other_struct.current_time))
    return false;
  return true;
}

template <typename T, MediaStatus::EnableIfSame<T>*>
bool operator<(const T& lhs, const T& rhs) {
  if (lhs.title < rhs.title)
    return true;
  if (rhs.title < lhs.title)
    return false;
  if (lhs.can_play_pause < rhs.can_play_pause)
    return true;
  if (rhs.can_play_pause < lhs.can_play_pause)
    return false;
  if (lhs.can_mute < rhs.can_mute)
    return true;
  if (rhs.can_mute < lhs.can_mute)
    return false;
  if (lhs.can_set_volume < rhs.can_set_volume)
    return true;
  if (rhs.can_set_volume < lhs.can_set_volume)
    return false;
  if (lhs.can_seek < rhs.can_seek)
    return true;
  if (rhs.can_seek < lhs.can_seek)
    return false;
  if (lhs.play_state < rhs.play_state)
    return true;
  if (rhs.play_state < lhs.play_state)
    return false;
  if (lhs.is_muted < rhs.is_muted)
    return true;
  if (rhs.is_muted < lhs.is_muted)
    return false;
  if (lhs.volume < rhs.volume)
    return true;
  if (rhs.volume < lhs.volume)
    return false;
  if (lhs.duration < rhs.duration)
    return true;
  if (rhs.duration < lhs.duration)
    return false;
  if (lhs.current_time < rhs.current_time)
    return true;
  if (rhs.current_time < lhs.current_time)
    return false;
  return false;
}


}  // namespace mojom
}  // namespace media_router

namespace mojo {


template <>
struct  StructTraits<::media_router::mojom::MediaStatus::DataView,
                                         ::media_router::mojom::MediaStatusPtr> {
  static bool IsNull(const ::media_router::mojom::MediaStatusPtr& input) { return !input; }
  static void SetToNull(::media_router::mojom::MediaStatusPtr* output) { output->reset(); }

  static const decltype(::media_router::mojom::MediaStatus::title)& title(
      const ::media_router::mojom::MediaStatusPtr& input) {
    return input->title;
  }

  static decltype(::media_router::mojom::MediaStatus::can_play_pause) can_play_pause(
      const ::media_router::mojom::MediaStatusPtr& input) {
    return input->can_play_pause;
  }

  static decltype(::media_router::mojom::MediaStatus::can_mute) can_mute(
      const ::media_router::mojom::MediaStatusPtr& input) {
    return input->can_mute;
  }

  static decltype(::media_router::mojom::MediaStatus::can_set_volume) can_set_volume(
      const ::media_router::mojom::MediaStatusPtr& input) {
    return input->can_set_volume;
  }

  static decltype(::media_router::mojom::MediaStatus::can_seek) can_seek(
      const ::media_router::mojom::MediaStatusPtr& input) {
    return input->can_seek;
  }

  static decltype(::media_router::mojom::MediaStatus::play_state) play_state(
      const ::media_router::mojom::MediaStatusPtr& input) {
    return input->play_state;
  }

  static decltype(::media_router::mojom::MediaStatus::is_muted) is_muted(
      const ::media_router::mojom::MediaStatusPtr& input) {
    return input->is_muted;
  }

  static decltype(::media_router::mojom::MediaStatus::volume) volume(
      const ::media_router::mojom::MediaStatusPtr& input) {
    return input->volume;
  }

  static const decltype(::media_router::mojom::MediaStatus::duration)& duration(
      const ::media_router::mojom::MediaStatusPtr& input) {
    return input->duration;
  }

  static const decltype(::media_router::mojom::MediaStatus::current_time)& current_time(
      const ::media_router::mojom::MediaStatusPtr& input) {
    return input->current_time;
  }

  static bool Read(::media_router::mojom::MediaStatus::DataView input, ::media_router::mojom::MediaStatusPtr* output);
};

}  // namespace mojo

#endif  // CHROME_COMMON_MEDIA_ROUTER_MOJO_MEDIA_STATUS_MOJOM_H_